
import java.util.*;
import java.io.*;
import jahuwaldt.tools.math.EvaluatableNumOpt;
import jahuwaldt.tools.GeneticSearch.*;


/**
*  A simple application to test my Genetic Search Algorithm routines.
*
*  <p>  Modified by:   Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt    Date:  May 26, 1999
*  @version  June 21, 2002
**/
public class GSADemo {

	public static void main( String[] args ) {

		System.out.println( "Start..." );

		// Create a "input" object to make it easier to pass a whole lot of inputs all at once.
		// when creating a new GeneticSearch object.
		GSAInputs inputs = new GSAInputs();
		inputs.functions = new MyProblem();
		inputs.populationSize = 50;
		inputs.bitsPerVariable = new int [ 4 ];
		inputs.bitsPerVariable[0] = 2;
		inputs.bitsPerVariable[1] = 20;
		inputs.bitsPerVariable[2] = 20;
		inputs.bitsPerVariable[3] = 20;
		inputs.lowerBounds = new float [ 4 ];
		inputs.lowerBounds[0] = 1;
		inputs.lowerBounds[1] = 0.0535F;
		inputs.lowerBounds[2] = 0.05F;
		inputs.lowerBounds[3] = 0.001F;
		inputs.upperBounds = new float [ 4 ];
		inputs.upperBounds[0] = 4;
		inputs.upperBounds[1] = 2;
		inputs.upperBounds[2] = 2;
		inputs.upperBounds[3] = 0.1F;
		inputs.orgFactory = new BinOrganismFactory();

		// Create a new GeneticSearch using the just created set of inputs.
		GeneticSearch gsa = new GeneticSearch( inputs );

		// Set the probability of mutation (default is 0.01).
		gsa.setMutationProb( 0.5F );

		// Determine initial minimum cost of random designs.
		float minCost = gsa.getMinimumCost();
		System.out.println( "Initial Population: Min Cost = " + minCost );

		// Evolve the population of designs for 500 generations.
		minCost = gsa.evolve( 500, true );
		System.out.println( "Final Population:  Min Cost = " + minCost );

		// Try serializing the network.
		try  {
			String filename = "gsa.out";

			// Create an object stream to the file.
			ObjectOutputStream out = new ObjectOutputStream( new FileOutputStream( filename ) );

			// Serialize the network.
			System.out.println( "\nSerializing the GSA state to a disk file..." );
			out.writeObject( gsa );
			out.close();

			// Remove our reference to the existing data.
			gsa = null;

			// Create an object stream from the file.
			ObjectInputStream in = new ObjectInputStream( new FileInputStream( filename ) );

			// Reconstitute the table.
			System.out.println( "Restoring the GSA state from disk file..." );
			gsa = (GeneticSearch) in.readObject();
			in.close();

			minCost = gsa.getMinimumCost();
			System.out.println( "Final Population:  Min Cost = " + minCost );

		} catch( Exception e ) {
			e.printStackTrace();
		}

		System.out.println( "Done" );
	}

}


/**
*  The problem to be optimized by the genetic algorithm.
*  The problem consists of a textbook structural airplane wing design
*  optimization.  I don't know any details about the problem except
*  that the optimum solution results in a minimum cost of about 7.5.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author   Joseph A. Huwaldt    Date:  May 26, 1999
*  @version  June 21, 2002
**/
class MyProblem extends EvaluatableNumOpt {

	// Force in pounds.
	private static final float Force = 3570.F;

	// Length in inches.
	private static final float L = 25.0F;

	// Width in inches.
	private static final float w = 10.0F;

	// Height in inches.
	private static final float h = 10.0F;

	/**
	*  The cost function for a text-book structural airplane wing
	*  design problem.
	*
	*  @param  x  Array (vector) of design variables to be used to
	*             calculate the cost of this design solution.
	*                 x[0] = nt:   Number of stringers.
	*                 x[1] = At:   Area of the stringers.
	*                 x[2] = Asc:  Area of the spar cap.
	*                 x[3] = t:    Skin thickness.
	*  @return  The cost (weight) of a given set of design variables.
	**/
	public float cost( float[] x ) {
		float nt = x[0];
		float At = x[1];
		float Asc = x[2];
		float t = x[3];

		//	Evaluate Cost Function.
		float value = 0.101F*L*(2*t*w + 4*Asc + 2*At*nt + 2*t*(h - 2*t));

		// Return cost of this set of design variables.
		return ((float) (value));
	}

	/**
	*  Calculates the penalty associated with violating the
	*  multivariable design constraints of the problem being
	*  optimized.
	*
	*  @param   x  Array (vector) of design variables to be used to
	*              calculate the penalty associated this design solution.
	*  @return  The penalty associated with this set of design variables.
	**/
	public float penalty( float[] x ) {
		float nt = x[0];
		float At = x[1];
		float Asc = x[2];
		float t = x[3];

		//	Constraints
		float g1, g2, g3, g4;

		//	Scaling for fn
		float R;
		float BS, K, B, S, FallA;
		float Atop, FcalA, FcalB;
		float FallB, Bstar, Fcrstr;
		float FallC, FcalC, Sstar;
		float FallD, FcalD;

		//	At limit load.
		BS = (w - 2*t - Asc)/(nt + 1);
		K = 5 - 4.23F/BS;
		B = (float)(BS/t/Math.sqrt( 10.7e6*K/39.0e3 ));
		S = 1.25F - 0.536F*B;
		FallA = S*39.0e3F;
		Atop = 2*Asc + nt*At + w*t + 1F/3*h*t;
		FcalA = Force*L/(Atop*h);

		//	At limit (shear).
		FcalB = Force/(2*h);
		FallB = 0.55F*64000*t;

		//	At 90% ultimate.
		Atop = Atop - 0.5F*w*t;
		Bstar = (float)(1/t/Math.sqrt(0.6*11.0e6/34.0e3));
		S = 1.25F - 0.536F*Bstar;
		Fcrstr = Math.abs( S*34.0e3F );
		B = (float)(L/1.2247/0.3/(Math.PI*Math.sqrt( 11.0e6/Fcrstr )));
		S = 1.25F - 0.536F*B;
		FallC = S*Fcrstr;
		FcalC = Force*L*1.5F*0.9F/(Atop*h);

		//	At 120% ultimate
		Atop = Atop - 0.2F*nt*At;
//		Bstar = (float)(1/t/Math.sqrt(0.6*11.0e6/34.0e3));
		Sstar = 1.25F - 0.536F*Bstar;
		Fcrstr = Sstar*34.0e3F;
		FallD = Fcrstr;
		FcalD = Force*L*1.5F*1.2F/(Atop*h);

		//	Formulate Constraints
		g1 = 1 - FallA / FcalA;
		g2 = 1 - FallB / FcalB;
		g3 = 1 - FallC / FcalC;
		g4 = 1 - FallD / FcalD;

		g1 = Math.max( 0, g1 );
		g2 = Math.max( 0, g2 );
		g3 = Math.max( 0, g3 );
		g4 = Math.max( 0, g4 );
		R = 500;

		//	Evaluate Penalty Function
		double value = R * (g1 * g1 + g2 * g2 + g3 * g3 + g4 * g4);

		return (float)value;
	}


}


