/*
*   BinChromosome  -- A type of GA Chromosome that uses binary encoding.
*
*   Copyright (C) 1999-2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
**/
package jahuwaldt.tools.GeneticSearch;

import java.util.*;


/**
*  A type of Chromosome for Genetic Algorithms that uses a simple binary encoding
*  of the parameter information in the chromosome.
*
*  <p> Modified by:  Joseph A. Huwaldt    </p>
*
*	@author    Joseph A. Huwaldt    Date:  May 23, 1999
*	@version   June 21, 2002
**/
public class BinChromosome implements Chromosome, SharedRandom {

	// The bitset provides the binary representation of a single gene or chromosome.
	private BitSet bits = null;

	// The length of the chromosome (the number of bits).
	private int numBits = 0;

	private float minValue = 0;

	private float maxValue = 1;

	private float curValue = 0;

	//-----------------------------------------------------------------------------
	/**
	*  Constructor used to generate a new binary chromosome of a
	*  given length. The chromosome will have a value of 0 stored 
	*  in it.
	*
	*  @param  The length of the new chromosome (the number of bits).
	**/
	public BinChromosome( int length ) {
	
		// Allocate memory for our bit field.
		bits = new BitSet( length );
		numBits = length;
		
	}

	/**
	*  Constructor used to generate a new binary chromosome of a
	*  given length with the range and current value specified.
	*
	*  @param  length  The length of the new chromosome (the number of bits).
	*  @param  min     The minimum value that can be encoded in this chromosome.
	*  @param  max     The maximum value that can be encoded in this chromosome.
	*  @param  value   The current value to store in this chromosome.
	**/
	public BinChromosome( int length, float min, float max, float value ) {

		// Allocate memory for our bit field.
		bits = new BitSet( length );
		numBits = length;

		// Set the initial value encoded in this chromosome.
		curValue = value;
		setRange( min, max );
	}

	//-----------------------------------------------------------------------------
	/**
	*  Method that changes the parameter value encoded in this chromosome.
	*
	*  @param  New value to be encoded in this chromosome.
	**/
	public void setValue( float input ) {
	
		if ( input <= minValue ) {
			clearAllBits();
			curValue = minValue;
			
		} else {
		
			if ( input >= maxValue ) {
				setAllBits();
				curValue = maxValue;
				
			} else {
				int maxDec = pow2( numBits ) - 1;
				float range = (maxValue - minValue);
				int decValue = Math.round( maxDec*(input - minValue)/range );

				dec2bin( decValue, bits );
				curValue = decValue*range/maxDec + minValue;
			}
		}
	}

	/**
	*  Returns the parameter value encoded in this chromosome.
	*
	*  @return  The parameter value recorded in this chromosome.
	**/
	public float getValue() {
		return curValue;
	}

	/**
	*  Method that changes the range of values that can be encoded
	*  in this chromosome.  If the current value stored in this
	*  chromosome is outside the min/max range specified here,
	*  it will be adjusted such that it is at either the min or
	*  max value.
	*
	*  @param  min  Minimum value that can be encoded in this chromosome.
	*  @param  max  Maximum value that can be encoded in this chromosome.
	**/
	public void setRange( float min, float max ) {
		minValue = min;
		maxValue = max;
		setValue( curValue );
	}

	/**
	*  Returns the minimum value that can be encoded in this chromosome.
	*
	*  @return  The minimum value that can be encoded in this chromosome.
	**/
	public float getMinValue() {
		return minValue;
	}

	/**
	*  Returns the maximum value that can be encoded in this chromosome.
	*
	*  @return  The maximum value that can be encoded in this chromosome.
	**/
	public float getMaxValue() {
		return maxValue;
	}

	/**
	*  Returns the length of this chromosome (the number of encoding elements it contains).
	*
	*  @return  The length of this chromosome.
	**/
	public int length() {
		return numBits;
	}

	/**
	*  Used to change the number of encoding elements in this chromosome.
	*
	*  @param  newLength  New number of encoding elements in this chromosome.
	**/
	public void setLength( int newLength ) {
		numBits = newLength;
		setValue( curValue );
	}

	/**
	*  Method that causes a single random mutation to occure in this chromosome.
	**/
	public void mutate() {
		float ranNum = ranNumGen.nextFloat();
		int picked = (int) (ranNum * numBits);

		// NOT (flip) the randomly picked bit.
		if ( bits.get( picked ) )
			bits.clear( picked );
		else
			bits.set( picked );

		// Update the current value to reflect the change.
		int maxDec = pow2( numBits ) - 1;
		int decValue = bin2dec( bits );
		curValue = decValue*(maxValue - minValue)/maxDec + minValue;
	}

	/**
	*  Make a copy of this chromosome.
	*
	*  @return Returns a clone of this chromosome.
	**/
	public Object clone() {
		BinChromosome newObject = null;
		
		try {
			// Make a shallow copy of this object.
			newObject = (BinChromosome)super.clone();
			
			// Now make deep copy of the data contained in this object.
			if ( bits != null )
				newObject.bits = (BitSet)bits.clone();

			// Output the new cloned neuron.
			
		} catch (CloneNotSupportedException e) { /* Can't happen */ }
		
		return newObject;
	}

	//-----------------------------------------------------------------------------
	/**
	*  Method that sets all the bits in the chromosome to false.
	*  This effectively sets the chromosome to the minimum value.
	**/
	protected void clearAllBits() {
		int length = bits.size();
		for ( int i = 0; i < length; ++i )
			bits.clear( i );
	}

	/**
	*  Method that sets all the bits in the chromosome to true.
	*  This effectively sets the chromosome to the maximum value.
	**/
	protected void setAllBits() {
		int length = numBits;
		for ( int i = 0; i < length; ++i )
			bits.set( i );
	}

	/**
	*  Method that raises 2 to the small integer power indicated (eg:  2^3 = 8).
	*
	*  @param  x  Amount to raise 2 to the power of.
	*  @return  Returns 2 raised to the power indicated.
	**/
	protected static final int pow2( int x ) {
		int value = 1;
		for ( int i = 0; i < x; ++i )
			value *= 2;
		return value;
	}

	/**
	*  Converts a positive decimal number to it's binary equivelant.
	*
	*  @param  decimal  The positive decimal number to be encoded in binary.
	*  @param  bits     The bitset to encode the number in.
	**/
	protected static final void dec2bin( int decimal, BitSet bits ) {
		if ( decimal < 0 )
			throw new IllegalArgumentException( "Cannot convert a negative number to binary." );
		
		int i = 0;
		int value = decimal;
		while ( value > 0 ) {
		
			if ( value % 2 > 0 )
				bits.set( i );
			else
				bits.clear( i );
			
			value /= 2;
			++i;
		}

		for ( i = i; i < bits.size(); ++i )
			bits.clear( i );
	}

	/**
	*  Converts binary number to it's base 10 decimal equivelant.
	*
	*  @param  bits  The bitset that encodes the number to be converted.
	*  @return  Returns the decimal equivelent of the given binary number.
	**/
	protected static final int bin2dec( BitSet bits ) {
		int value = 0;
		int length = bits.size();

		for ( int i = 0; i < length; ++i ) {
			if ( bits.get( i ) )
				value += pow2( i );
		}
		return value;
	}

}


