/*
*   BinOrganism  -- A type of GA Organism that uses binary encoded chromosomes.
*
*   Copyright (C) 1999-2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
**/
package jahuwaldt.tools.GeneticSearch;

import java.util.*;


/**
*  A type of Organism for Genetic Algorithms that uses a simple binary encoding
*  of the parameter information in it's chromosomes.  Each design variable is
*  encoded in it's own chromosome and during crossover, design variables are swapped
*  but the bits in individual design variables are not crossed (a design variable is
*  treated as an "atomic" indivisable whole).
*
*  <p> Modified by:  Joseph A. Huwaldt    </p>
*
*  @author    Joseph A. Huwaldt    Date:  May 24, 1999
*  @version   June 21, 2002
**/
public class BinOrganism implements Organism, SharedRandom {

	// Genetic information is recorded as an array of chromosomes.
	// Each array member corresponds to a design variable.
	private  BinChromosome[] genes = null;

	//-----------------------------------------------------------------------------
	/**
	*  Constructor that creates a binary GSA organism with a
	*  specified number of genes (design variables) of specified
	*  lengths (resolution of each design variable).
	*
	*  @param  lengths  Arrray (vector) indicating the number of bits
	*                   per design variable.
	**/
	public BinOrganism( int[] lengths ) {

		int numGenes = lengths.length;

		// Allocate memory for the array of genes.
		genes = new BinChromosome [ numGenes ];

		for ( int i = 0; i < numGenes; ++i )
			genes[i] = new BinChromosome( lengths[i] );
		
	}

	//-----------------------------------------------------------------------------
	/**
	*  Method used to set the vector of parameter values encoded in this organism.
	*
	*  @param  newValues  Array of parameter values to be encoded in this organism.
	**/
	public void setValues( float[] newValues ) {
		if ( newValues.length != genes.length )
			throw new IllegalArgumentException( "Number of values does not match number of genes." );
		
		for ( int i = 0; i < newValues.length; ++i )
			genes[i].setValue( newValues[i] );
	}

	/**
	*  Returns the vector of parameters encoded in this organism.
	*  The output is delivered via a reference to an already existing
	*  array in order to improve efficiency (rather than creating a
	*  new array every time) for this commonly  called routine.  
	*
	*  @param  A reference to an already existing array that will be
	*          filled in with the values encoded in this organism.
	*  @return  Returns the vector of parameter values encoded in this
	*           organism via reference.
	**/
	public void getValues( float[] values ) {
		if ( values.length != genes.length )
			throw new IllegalArgumentException( "Number of values does not match number of genes." );

		for ( int i = 0; i < values.length; ++i )
			values[i] = genes[i].getValue();
	}

	/**
	*  Method that sets the minimum and maximum values (the range of
	*  values) that can be encoded in each parameter in this organism.
	*
	*  @param  minValues  The vector of minimum values for each param that
	*                     can be stored in this organism.
	*  @param  maxValues  The vector of max values for each parameter that
	*                     can be stored in this organism.
	**/
	public void setRanges( float[] minValues, float[] maxValues ) {
		if ( minValues.length != genes.length || maxValues.length != genes.length )
			throw new IllegalArgumentException( "Range array sizes do not match number of genes." );

		for ( int i = 0; i < minValues.length; ++i )
			genes[i].setRange( minValues[i], maxValues[i] );
	}

	/**
	*  Return the vector of minimum values for each parameter that
	*  can currently  be stored in this organism.
	*
	*  @return The min values that can be stored for each parameter in
	*          this organism.
	**/
	public float[] getMinValues() {
		float[] values = new float [ genes.length ];

		for ( int i = 0; i < genes.length; ++i )
			values[i] = genes[i].getMinValue();
		
		return values;
	}

	/**
	*  Returns the vector of maximum values for each parameter that
	*  can currently be stored in this organism.
	*
	*  @return  The max values that can be stored for each parameter in
	*           this organism.
	**/
	public float[] getMaxValues() {
		float[] values = new float [ genes.length ];

		for ( int i = 0; i < genes.length; ++i )
			values[i] = genes[i].getMaxValue();
		
		return values;
	}

	/**
	*  Method that causes a mutation to occure at a random location
	*  in this organism's chromosomes.
	**/
	public void mutate() {
		float ranNum = ranNumGen.nextFloat();

		// Pick a gene to mutate.
		int picked = (int)(ranNum*genes.length);

		// Cause a mutation to occure in the picked gene.
		genes[picked].mutate();
	}

	/**
	*  Method that mates or crosses two organisms together producing a new
	*  organism that has some genetic information from one parent and
	*  some from the other.
	*
	*  @param  The organism that this organism is to be mated with.
	*  @return The child or offspring organism.  A new organism with some 
	*          of the genetic material from each of the parents.
	**/
	public Organism mate( Organism partner ) {
		int i;
		BinOrganism child = null;
		BinOrganism thePartner = (BinOrganism)partner;
		
		// Start with a copy of this organism.
		child = (BinOrganism)this.clone();

		// Pick two random places on the genome to crossover between.
		int length = genes.length;
		float ranNum = ranNumGen.nextFloat();
		int pos1 = (int) (ranNum*length);
		ranNum = ranNumGen.nextFloat();
		int pos2 = (int) (ranNum*length);

		// Copy the genes between pos1 and pos2 from the partner to the child organisms.
		if ( pos1 <= pos2 ) {
			// The two positions do not include the ends of the chromosome.
			for ( i = pos1; i <= pos2; ++i )
				child.genes[i] = (BinChromosome)thePartner.genes[i].clone();
				
		} else {
			// The two positions do include the ends of the chromosome.
			for ( i = pos1; i < length; ++i )
				child.genes[i] = (BinChromosome)thePartner.genes[i].clone();
			
			for ( i = 0; i <= pos2; ++i )
				child.genes[i] = (BinChromosome)thePartner.genes[i].clone();
		}

		return child;
	}

	/**
	*  Make a copy of this organism.
	*
	*	@return  Returns a clone of this organism.
	**/
	public Object clone() {
		BinOrganism newObject = null;
		
		try {
			// Make a shallow copy of this object.
			newObject = (BinOrganism) super.clone();

			// Now make deep copy of the data contained in this object.
			if ( genes != null ) {
				int length = genes.length;
				newObject.genes = new BinChromosome[ length ];

				for ( int i = 0; i < length; ++i )
					newObject.genes[i] = (BinChromosome)genes[i].clone();
			}
			
		} catch (CloneNotSupportedException e) { /* Can't happen. */ }
		
		// Output the new cloned neuron.
		return newObject;
	}

}


