/*
*   Chromosome  -- Interface for all GA Chromosome type objects.
*
*   Copyright (C) 1999-2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
**/
package jahuwaldt.tools.GeneticSearch;

import java.util.*;


/**
*  Interface in common for all chromosome types used in Genetic Algorithms.
*  Typically, you will create an "Organism" object that contains a chromosome
*  or series of chromosomes (genes) that encode the genetic information.
*
*  <p> Modified by:  Joseph A. Huwaldt    </p>
*
*  @author    Joseph A. Huwaldt    Date:  May 23, 1999
*  @version   June 21, 2002
**/
public interface Chromosome extends Cloneable, java.io.Serializable {

	/**
	*  Method that changes the parameter value encoded in this chromosome.
	*
	*  @param  New value to be encoded in this chromosome.
	**/
	void setValue( float input );

	/**
	*  Returns the parameter value encoded in this chromosome.
	*
	*  @return  Returns the parameter value recorded in this chromosome.
	**/
	float getValue();

	/**
	*  Method that changes the range of values that can be encoded
	*  in this chromosome.  If the current value stored in this
	*  chromosome is outside the min/max range specified here,
	*  it will be adjusted such that it is at either the min or max value.
	*
	*  @param  min  Minimum value that can be encoded in this chromosome.
	*  @param  max  Maximum value that can be encoded in this chromosome.
	**/
	void setRange( float min, float max );

	/**
	*  Returns the minimum value that can be encoded in this chromosome.
	*
	*  @return  Returns minimum value that can be encoded in this
	*           chromosome.
	**/
	float getMinValue();

	/**
	*  Returns the maximum value that can be encoded in this chromosome.
	*
	*  @return  Returns maximum value that can be encoded in this chromosome.
	**/
	float getMaxValue();

	/**
	*  Returns the length of this chromosome (the number of encoding elements it contains).
	*
	*  @return  Returns the length of this chromosome.
	**/
	int length();

	/**
	*  Method that changes the number of encoding elements in this chromosome.
	*
	*  @param  New number of encoding elements in this chromosome.
	**/
	void setLength( int newLength );

	/**
	*  Method that causes a single mutation to occure in this chromosome.
	**/
	void mutate();

	/**
	*  Make a copy of this chromosome.
	*
	*  @return Returns a clone of this chromosome.
	**/
	Object clone();
	
}

interface SharedRandom {
	Random ranNumGen = new Random();
}


