/*
*   GeneticSearch  -- A simple genetic search algorithm for numerical optimization.
*
*   Copyright (C) 1999-2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
**/
package jahuwaldt.tools.GeneticSearch;

import jahuwaldt.tools.math.EvaluatableNumOpt;


/**
*  A simple genetic search algorithm for numerical optimization.
*
*  <p> Modified by:  Joseph A. Huwaldt    </p>
*
*  @author    Joseph A. Huwaldt    Date:  May 24, 1999
*  @version   June 21, 2002
**/
public class GeneticSearch implements SharedRandom, java.io.Serializable {

	// The current population of organisms (design solutions).
	private Organism[] population = null;

	private int popSize;

	// The fitness and cost values associated with each member of the current population.
	private float[] fitness = null;

	private float[] cost = null;

	private float fitSum;

	// References to the cost and penalty functions.
	private EvaluatableNumOpt functions = null;

	private int numVars;

	// Probability that crossover will occure.
	private float pc = 0.7F;

	// Probability that mutation will occure.
	private float pm = 0.01F;

	// Index into population of the best (most fit) individual organism.
	private int best = -1;

	//-----------------------------------------------------------------------------
	/**
	*  Constructor that sets up a genetic search for numerical optimization.
	*
	*  @param  functions  A reference to a class that contains a cost
	*                     function and a penalty function used to evaluate a given
	*                     set of design variables.
	*  @param  populationSize  The number of organisms to include in the design search.
	*  @param  bitsPerVariable The number of bits to use for each design variable.
	*  @param  lowerBounds     The lower bounds (smallest value) for each design variable.
	*  @param  upperBounds     The upper bounds (largest value) for each design variable.
	*  @param  theFactory      A class that is used to generate whatever type of organism
	*                          you want to use to facilitate the genetic search.
	**/
	public GeneticSearch( EvaluatableNumOpt functions, int populationSize,
							int[] bitsPerVariable, float[] lowerBounds,
							float[] upperBounds, OrganismFactory theFactory ) {

		setup( functions, populationSize, bitsPerVariable, lowerBounds,  upperBounds, theFactory );
	}

	/**
	*  Constructor that sets up a genetic search for numerical
	*  optimization using an input class.
	*
	*  @param  inputs  A GSAInput class containing the inputs required for
	*                  this genetic search.
	**/
	public GeneticSearch( GSAInputs inputs ) {

		setup( inputs.functions, inputs.populationSize, inputs.bitsPerVariable, inputs.lowerBounds,
					   inputs.upperBounds, inputs.orgFactory );
	}


	//-----------------------------------------------------------------------------
	private void setup( EvaluatableNumOpt functions, int populationSize,
							int[] bitsPerVariable, float[] lowerBounds,
							float[] upperBounds, OrganismFactory theFactory ) {
		int i;
		float range, xLB;
		float[] values = null;

		// Check for consistant inputs.
		numVars = bitsPerVariable.length;
		if ( lowerBounds.length != numVars || upperBounds.length != numVars )
			throw new IllegalArgumentException( "Input arrays have inconsistant sizes." );
		
		// Initialize some variables.
		popSize = populationSize;
		this.functions = functions;

		// Allocate memory for cost & fitness information.
		cost = new float [ populationSize ];
		fitness = new float [ populationSize ];

		// Create an initial popluation of organisms (design solutions).
		population = new Organism [ populationSize ];
		for ( i = 0; i < populationSize; ++i ) {
			population[i] = theFactory.newOrganism( bitsPerVariable );
			population[i].setRanges( lowerBounds, upperBounds );
		}

		// Give initial population random combinations of design variables.
		values = new float [ numVars ];
		for ( i = 0; i < populationSize; ++i ) {
			for ( int j = 0; j < numVars; ++j ) {
				xLB = lowerBounds[j];
				range = upperBounds[j] - xLB;
				values[j] = ranNumGen.nextFloat() * range + xLB;
			}
			population[i].setValues( values );
		}

		// Evaluate the fitness of the initial population ("values" is used as storage space only, content is irrelevant).
		evaluateOrganisms( values );
	}

	/**
	*	Returns the probability of genetic crossover occuring during
	*   each reproduction cycle.  The returned value is a number
	*   between 0. 0 (never) and 1.0 (always).
	*
	*	@return	Returns the probability of genetic crossover occuring.
	**/
	public float getCrossoverProb() {
		return pc;
	}

	/**
	*	Method that sets the probability of genetic crossover occuring during each
	*   reproduction cycle.  The input value is a number between
	*   0.0 (never) and 1.0 (always).  The default value is 0.70.
	*
	*	@param	The new probability of genetic crossover occuring.
	**/
	public void setCrossoverProb( float probability ) {
		pc = probability;
	}

	/**
	*	Returns the probability of genetic mutation occuring during each
	*   reproduction cycle.  The returned value is a number between
	*   0.0 (never) and 1.0 (always).
	*
	*	@return	Returns the probability of genetic mutation occuring.
	**/
	public float getMutationProb() {
		return pm;
	}

	/**
	*	Method that sets the probability of genetic mutation occuring during each
	*   reproduction cycle.  The input value is a number between
	*   0. 0 (never) and 1.0 (always).  The default value is 0.01.
	*
	*	@param	The new probability of genetic mutation occuring.
	**/
	public void setMutationProb( float probability ) {
		pm = probability;
	}

	/**
	*	Method that finds the design with the smallest cost + penalty value (highest
	*   fitness)  by evolving organisms that encode design variables.
	*   This version of "evolve" allows you to specify the number of
	*   generations to evolve.
	*
	*  @param numGenerations  Number of generations to evolve the organisms encoding
	*                         the designs.
	*  @param status          Report the min cost of each generation if true.
	*  @return  Returns the cost of the best design found.
	**/
	public float evolve( int numGenerations, boolean status ) {
		int i;
		float[] pmate = new float [ popSize ];
		Organism[] nextGeneration = new Organism [ popSize ];
		
		// Allocate storage space for design variable values here.
		float[] values = new float [ numVars ];

		//  Loop over each generation requested.
		for ( int gen = 0; gen < numGenerations; ++gen ) {

			// Determine mating probability (roulette wheel area) of each organism.
			for ( i = 0; i < popSize; ++i )
				pmate[i] = fitness[i] / fitSum;

			// Find pairs of designs and mate them.
			reproduce( pmate, nextGeneration );

			// Completely replace the old generation with the new one.
			for ( i = 0; i < popSize; ++i )
				population[i] = nextGeneration[i];

			// Evaluate the new generation of organisms
			// ("values" is passed as storage space only, content will be overwritten).
			evaluateOrganisms( values );

			if ( status )
				System.out.println( "Min Cost for gen[" + gen + "] = organism[" + best + "] = " + cost[best] );
		}
		
		return cost[best];
	}


	/**
	*  Method that returns the minimum cost of any organism in the population.
	*  This is the cost of the most fit organism.
	**/
	public float getMinimumCost() {
		return cost[best];
	}
	
	
	/**
	*  Method that determines the fitness of each organism in the population by
	*  calling the cost and penalty functions with the each organism's
	*  design variables.
	*
	*  @param  values  Pre-allocated storage space for design variables.
	*                  This space is allocated outside of "evaluateOrganisms" to
	*                  boost performance (so it doesn't have to be reallocated for
	*                  each generation) but is used only inside of this routine.
	*                  Any values stored in this array will be overwritten and the
	*                  values contained on completion of the routine are meaningless.
	**/
	protected void evaluateOrganisms( float[] values ) {
		int i;
		float penalty, ppcMax = 0;
		float bestFit = 0;

		// Evaluate the cost of each design solution (organism) by calling the cost
		// and penalty functions.
		for ( i = 0; i < popSize; ++i ) {
		
			// Extract the design variables stored in each organism.
			population[i].getValues( values );

			// Evaluate the cost and penalty functions.
			cost[i] = (float) functions.cost( values );
			penalty = functions.penalty( values );

			// Combine the cost & penalty values and keep track of the maximum.
			fitness[i] = cost[i] + penalty;
			ppcMax = Math.max( ppcMax, cost[i] + penalty );
		}

		// Up the max penalty + cost values by 1%.
		ppcMax += ppcMax * 0.01;
		fitSum = 0;

		// Convert cost+penalty values to fitness & ID the best.
		for ( i = 0; i < popSize; ++i ) {
			fitness[i] = ppcMax - fitness[i];
			fitSum += fitness[i];

			if ( fitness[i] > bestFit ) {
				bestFit = fitness[i];
				best = i;
			}
		}

	}

	/**
	*  Method that handles picking organisms and mating them to create offspring.
	*
	*	@param  wheelAreas  An array of values that are proportional to the
	*                       fitness of each organism in the population (all the values
	*                       together must add up to 1.0).  Used with roulette wheel
	*                       algorithm to pick organisms for mating.
	*  @param  nextGeneration  An array that will be filled in with the
	*                          offspring of the mating process.
	**/
	protected void reproduce( float[] wheelAreas, Organism[] nextGeneration ) {
		int i = 1;
		int org1, org2;

		// Retain the most fit organism without modification to prevent the min
		// cost design in population from accidentally increasing.
		nextGeneration[0] = population[best];

		// Keep the size of the population constant.
		do {
			// Pick the 1st organism.
			org1 = rouletteWheel( wheelAreas );

			// Are we going to mate this organism with another one?
			if ( ranNumGen.nextFloat() <= pc ) {
				// Pick a second organism and mate the two.
				org2 = rouletteWheel( wheelAreas );
				nextGeneration[i] = population[org1].mate( population[org2] );
				
			} else
				// No mating takes place, just copy the organism to the next generation.
				nextGeneration[i] = population[org1];

			// Are we going to mutate this child organism?
			if ( ranNumGen.nextFloat() <= pm ) {

				// Make sure we don't mutate the best organism in the population.
				if ( nextGeneration[i] == nextGeneration[0] )
					nextGeneration[i] = (Organism)nextGeneration[i].clone();

				// Mutate this organism.
				nextGeneration[i].mutate();
			}
			++i;
			
		} while ( i < popSize );
	}

	/**
	*  Picks an organism from the population using a roulette wheel
	*  algorithm.  Each organism has a probability of being picked
	*  that is passed to this routine in the form of a wheel area.  The
	*  more wheel area, the more likely it is that the organism will
	*  be picked.
	*
	*  @param  wheelAreas  An array of values that are proportional to the
	*                      fitness of each organism in the population (all the values
	*                      together must add up to 1.0).
	*  @return  Returns the index to the organism that has been picked.
	**/
	protected static int rouletteWheel( float[] wheelAreas ) {
		int popSize = wheelAreas.length;
		float ranNum = ranNumGen.nextFloat();
		
		// Pick a random starting point on the wheel.
		int i = (int) (ranNum * popSize);
		float warea = wheelAreas[i];

		//  Spin the wheel until the amount of wheel that has passed is greater than ranNum.
		ranNum = ranNumGen.nextFloat();
		while ( ranNum > warea ) {
			++i;
			if ( i >= popSize )
				i = 0;
			
			warea += wheelAreas[i];
		}

		// Return the index of the organism at the point where the wheel stopped.
		return i;
	}

}


