/*
*   Organism  -- Interface in common for all GA organism types.
*
*   Copyright (C) 1999-2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
**/
package jahuwaldt.tools.GeneticSearch;


/**
*  Interface for all organism types used in Genetic Algorithm numerical
*  optimization codes.
*
*  <p> Modified by:  Joseph A. Huwaldt    </p>
*
*  @author    Joseph A. Huwaldt    Date:  May 23, 1999
*  @version   June 21, 2002
**/
public interface Organism extends Cloneable, java.io.Serializable {

	/**
	*  Method that sets the vector of parameter values encoded in this organism.
	*
	*  @param  newValues  Array of parameter values to be encoded in the organism.
	**/
	void setValues( float[] newValues );

	/**
	*  Returns the vector of parameter encoded in this organism.
	*  The output is delivered via a reference to an already existing
	*  array in order to improve efficiency (rather than creating a
	*  new array every time) for this commonly  called routine.  
	*
	*  @param  values  A reference to an already existing array that will be
	*                  filled in with the values encoded in this organism.
	*  @return Returns the vector of parameter values encoded in this
	*          organism via reference.
	**/
	void getValues( float[] values );

	/**
	*  Method that sets the minimum and maximum values (the range of
	*  values) that can be encoded in each parameter in this organism.
	*
	*  @param  minValues  The vector of minimum values for each param that
	*                     can be stored in this organism.
	*  @param  maxValues  The vector of max values for each parameter that
	*                     can be stored in this organism.
	**/
	void setRanges( float[] minValues, float[] maxValues );

	/**
	*  Returns the vector of minimum values for each parameter that
	*  can currently  be stored in this organism.
	*
	*  @return  The min values that can be stored for each parameter in this organism.
	**/
	float[] getMinValues();

	/**
	*  Returns the vector of maximum values for each parameter that
	*  can currently be stored in this organism.
	*
	*  @return  The max values that can be stored for each parameter in
	*           this organism.
	**/
	float[] getMaxValues();

	/**
	*  Method that causes a mutation to occure at a random location on this
	*  organism's chromosomes.
	**/
	void mutate();

	/**
	*  Method that mates or crosses two organisms together producing a new
	*  organism that has some genetic information from one parent and
	*  some from the other.
	*
	*  @param  partner  The organism that this organism is to be mated with.
	*  @return The child or offspring organism.  A new organism with some of the
	*          genetic material from each of the parents.
	**/
	Organism mate( Organism partner );

	/**
	*  Make a copy of this organism.
	*
	*	@return	Returns a clone of this organism.
	**/
	Object clone();

}


