/*
*   EvaluatableNumOpt  -- Defines a multivariable cost function for
*                         numerical optimization.
*
*   Copyright (C) 1999-2002 by Joseph A. Huwaldt <jhuwaldt@knology.net>.
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
**/
package jahuwaldt.tools.math;


/**
*  <p>  A class that defines a multivariable cost function named
*       "cost" that can be used by numerical optimization routines
*       to return the cost associated with a particular set of
*       design variables (x[]).   This class also optionally defines
*       a penalty function named "penalty" that can be used to
*       return the penalty associated with violating a constraint
*       in a numerical optimization problem.  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  May 24, 1999
*  @version  June 21, 2002
**/
public abstract class EvaluatableNumOpt implements java.io.Serializable {

	/**
	*  <p>  User supplied method that calculates the multivariable
	*       cost function y = fn(x[]).
	*       Classes implementing this interface must define this
	*       function.
	*  </p>
	*
	*  @param   x  Array (vector) of design variables to be used to calculate
	*              the cost of this design solution.
	*  @return  Return the cost of this set of design variables.
	**/
	public abstract  float cost( float[] x ) ;

	/**
	*  <p>  Calculates the penalty associated with violating the
	*       multivariable design constraints of the problem being
	*       optimized.  Classes wanting to use a penalty function
	*       should override this function.  If the method using
	*       this class doesn't require a penalty function, do
	*       nothing and zero is returned (no penalty).  </p>
	*
	*  @param   x  Array (vector) of design variables to be used to
	*              calculate the penalty associated this design solution.
	*  @return  Return the penalty associated with this set of design
	*           variables.
	**/
	public  float penalty( float[] x ) {
		return 0.F;
	}


}


